from pathlib import Path, PurePosixPath
import pandas as pd
from typing import List, Tuple
from rich import print
from PIL import Image
import markdown
from weasyprint import HTML
import fitz
import html2text
from sleepyconvert_toolchain.params import *


def errorMessage(message:str) -> None:
  '''log error message'''
  print(f'[bold][red]Error[/bold]. {message}[/red]')


def successMessage(input_path:str, output_path:str, compress:bool) -> None:
  '''log success message'''
  output_path_display = f'{output_path}.gz' if compress else output_path
  print(f'✅ Converted [bold green]{input_path}[/bold green] ──▶ [bold green]{output_path_display}[/bold green]')


def verifyPaths(input_path:str, output_path:str, supported_formats:List[str]) -> Tuple[str, str]:
  '''check if input and output paths are valid, pass back formats'''
  path_object = Path(input_path)
  input_format = PurePosixPath(input_path).suffix.lower()[1:]
  output_format = PurePosixPath(output_path).suffix.lower()[1:]
  if not path_object.exists():
    errorMessage(f'Path {input_path} does not exist.')
    return '', ''
  elif not path_object.is_file():
    errorMessage(f'Path {input_path} is not a file.')
    return '', ''
  elif input_format not in supported_formats:
    errorMessage(f'Input format "{input_format}" not supported, must be one of: {", ".join(supported_formats)}')
    return '', ''
  elif output_format not in supported_formats:
    errorMessage(f'Output format "{output_format}" not supported, must be one of: {", ".join(supported_formats)}')
    return '', ''
  elif input_format == output_format:
    errorMessage(f'Input and output formats are the same: {input_format}')
  return input_format, output_format


def readData(format:str, path:str) -> pd.DataFrame:
  '''dispatch table to read data file according to file extension'''
  return {
    'csv':pd.read_csv,
    'json':pd.read_json,
    'parquet':pd.read_parquet,
    'pkl':pd.read_pickle,
    'xlsx':pd.read_excel,
  }[format](path)


def writeData(df:pd.DataFrame, format:str, path:str, compress:bool) -> None:
  '''dispatch table to write data file according to file extension'''
  dispatch = {
    'csv':df.to_csv,
    'json':df.to_json,
    'parquet':df.to_parquet,
    'pkl':df.to_pickle,
    'xlsx':df.to_excel,
  }
  if compress:
    dispatch[format](f'{path}.gz', compression='gzip')
  else:
    dispatch[format](path)


def convertPNGtoJPG(png_path:str, jpg_path:str, quality:int=100) -> None:
  """converts a PNG image to JPG format"""
  img = Image.open(png_path)
  if img.mode == "RGBA":
    img = img.convert("RGB")
  img.save(jpg_path, "JPEG", quality=quality)


def convertJPGtoPNG(jpg_path:str, png_path:str) -> None:
  """converts a JPG image to PNG format"""
  img = Image.open(jpg_path)
  img.save(png_path, "PNG")


def convertHTMLtoPDF(html_path:str, pdf_path:str) -> None:
  """converts an HTML file to PDF format"""
  HTML(html_path).write_pdf(pdf_path)


def convertPDFtoHTML(pdf_path:str, html_path:str) -> None:
  """converts a PDF file to HTML format"""
  pdf_document = fitz.open(pdf_path)
  html_content = ""
  for page_num in range(len(pdf_document)):
    page = pdf_document.load_page(page_num)
    html_content += page.get_text("html")
  with open(html_path, 'w', encoding='utf-8') as html_file:
    html_file.write(html_content)


def convertMDtoPDF(md_path:str, pdf_path:str) -> None:
  """converts a Markdown file to PDF format"""
  with open(md_path, 'r', encoding='utf-8') as f:
    markdown_text = f.read()
  html = markdown.markdown(markdown_text)
  HTML(string=html).write_pdf(pdf_path)


def convertPDFtoMD(pdf_path:str, md_path:str) -> None:
  """converts a PDF file to Markdown format"""
  pdf_document = fitz.open(pdf_path)
  markdown_content = ""
  for page_num in range(len(pdf_document)):
    page = pdf_document.load_page(page_num)
    text = page.get_text()
    # basic attempt to add headings based on potential font size (very rudimentary)
    lines = text.splitlines()
    for line in lines:
      if line.strip():
        if len(line.split()) < 6 and line[0].isupper(): # Very basic heuristic
          markdown_content += f"\n## {line.strip()}\n"
        else:
          markdown_content += f"{line.strip()}\n"
    markdown_content += "\n---\n" # Page separator

  with open(md_path, 'w', encoding='utf-8') as md_file:
    md_file.write(markdown_content)


def convertHTMLtoMD(html_path:str, md_path:str) -> None:
  """converts an HTML file to Markdown format"""
  with open(html_path, 'r', encoding='utf-8') as f:
    html_content = f.read()
  markdown_text = html2text.html2text(html_content)
  with open(md_path, 'w', encoding='utf-8') as md_file:
    md_file.write(markdown_text)


def convertMDtoHTML(md_path:str, html_path:str) -> None:
  """converts a Markdown file to HTML format"""
  with open(md_path, 'r', encoding='utf-8') as f:
    markdown_text = f.read()
  html = markdown.markdown(markdown_text)
  with open(html_path, 'w', encoding='utf-8') as html_file:
    html_file.write(html)



#!/usr/bin/env python

'''README

Usage:
  ./sleepyconvert.py data <input_path> <output_path> [options]
  ./sleepyconvert.py img <input_path> <output_path>
  ./sleepyconvert.py doc <input_path> <output_path>
'''

from sleepyconvert_toolchain.commands import app

if __name__ == "__main__":
  app()



from sys import exit
from rich import print
import typer
from sleepyconvert_toolchain.params import *
from sleepyconvert_toolchain.utils import *

app = typer.Typer(help="Convert common filetypes and data formats quickly.")


@app.command()
def data(input_path:str, output_path:str, compress:bool=False):
  '''Convert data file from one format to another, optionally compressing output'''
  input_format, output_format = verifyPaths(input_path, output_path, supported_data_formats)
  if not (input_format and output_format):
    exit(1)
    
  # additional guards
  if compress and output_format == 'xlsx':
    errorMessage('Cannot compress xlsx files. Please remove the --compress flag.')
    exit(1)

  # read
  try:
    df = readData(input_format, input_path)
  except Exception as e:
    errorMessage(f'Error reading data:\n{e}')
    exit(1)

  # write
  try:
    writeData(df, output_format, output_path, compress)
  except Exception as e:
    errorMessage(f'Error writing data:\n{e}')
    exit(1)

  # log
  output_path_display = f'{output_path}.gz' if compress else output_path
  successMessage(input_path, output_path_display, compress)
  exit(0)


@app.command()
def img(input_path:str, output_path:str):
  '''Convert image file from one format to another'''
  input_format, output_format = verifyPaths(input_path, output_path, supported_img_formats)
  if not (input_format and output_format):
    exit(1)

  # write
    try:
      match (input_format, output_format):
        case ('png', 'jpg'|'jpeg'):
          convertPNGtoJPG(input_path, output_path)
        case ('jpg'|'jpeg', 'png'):
          convertJPGtoPNG(input_path, output_path)
    except Exception as e:
      errorMessage(f'Error converting {input_format} to {output_format}:\n{e}')
      exit(1)

  # log
  successMessage(input_path, output_path, compress=False)
  exit(0)


@app.command()
def doc(input_path:str, output_path:str):
  '''Convert document file from one format to another'''
  input_format, output_format = verifyPaths(input_path, output_path, supported_doc_formats)
  if not (input_format and output_format):
    exit(1)

  # write
  try:
    match (input_format, output_format):
      case ('html', 'pdf'):
        convertHTMLtoPDF(input_path, output_path)
      case ('pdf', 'html'):
        convertPDFtoHTML(input_path, output_path)
      case ('md', 'pdf'):
        convertMDtoPDF(input_path, output_path)
      case ('pdf', 'md'):
        convertPDFtoMD(input_path, output_path)
      case ('html', 'md'):
        convertHTMLtoMD(input_path, output_path)
      case ('md', 'html'):
        convertMDtoHTML(input_path, output_path)
  except Exception as e:
    errorMessage(f'Error converting {input_path} to {output_path}:\n{e}')
    exit(1)

  # log
  successMessage(input_path, output_path, compress=False)
  exit(0)
